/*
 *   I2O Configuration utility
 *
 *   Software download
 *
 *   (C) Copyright 1999 University of Helsinki,
 *                      Department of Computer Science
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation; either version
 *   2 of the License, or (at your option) any later version.
 *
 *   Author     Taneli Vhkangas <Taneli.Vahakangas@cs.Helsinki.FI>
 *
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>     
#include <sys/ioctl.h>
#include <ctype.h>

#include <linux/i2o-dev.h>

#include "common.h"

#define FRAGSIZE 8192
#define MAXLEN 8192

int main(void)
{
	struct i2o_sw_xfer sw;
	struct stat sw_stat;
	size_t swlen;
	struct I2O_MODULE_DESC_HDR hdr;
	int iop, fd, ddmfile;
	unsigned int maxfrag = 0, curfrag = 0;
	char *query, *targetIOP, *moduleType, *source, *loadType, *overrideMode;
	struct page *p;
	void *buffer;

	p = (struct page *)malloc(sizeof(struct page));
	if (!p) {
		error_page("<H2>ERROR: Out of memory</H2><P>%s.\n", strerror(errno));
		return 1;
	}
	
	/* Decode the form results */ 
	
	query = getquery();
	targetIOP    = decode(extract(query, "TargetIOP"));
	moduleType   = decode(extract(query, "ModuleType"));
	source       = decode(extract(query, "Source"));
	loadType     = decode(extract(query, "LoadType"));
	overrideMode = decode(extract(query, "OverrideMode"));

 	fd = open ("/dev/i2o/ctl", O_RDONLY);
	if (fd < 0)
       	{
       		start_page(p);
		hprintf(p, "<H2>ERROR: Could not open connection to I<SUB>2</SUB>O Platform</H2>\n"
			"<P>%s\n", strerror(errno));
		end_page(p);
		return 1;
        }

	start_page(p);
	iop = atoi(targetIOP);
	hprintf(p, "<HTML>");
	hprintf(p, "<HEAD><TITLE>I2O Sofware Download</TITLE></HEAD>\n");
	hprintf(p, "<BODY>\n");
	hprintf(p, "<CENTER><H1>I<SUB>2</SUB>O Software download on IOP%d</H1></CENTER>\n", iop);

	hprintf(p, "<P>\n");
	if (strcmp(source,"") == 0) /* no file selected */
	{
		empty_page(p);
		hprintf(p, "<CENTER><H1><FONT COLOR=\"#F02020\">Error</FONT></H1>\n"
				"<P>You must select a file to download..."
				"<A HREF=\"" CGIPATH "IOPDetails?IOP=%d\">Back to start</a>", iop);
		end_page(p);
		close(fd);
		return 1;
	}

	hprintf(p, "<CENTER><H1>DDM Download</H1></CENTER>\n");
	hprintf(p, "<P>Using DDM file %s on \n", source);

	if (access(source, R_OK) <0 )
	{
		empty_page(p);
		hprintf(p, "<H2>ERROR: I don't have permission to read %s!</H2>\n"
				"<P>Does the file exist?\n", source);
		end_page(p);
		close(fd);
		return 1;
	}

	ddmfile = open(source, O_RDONLY, 0);
	if (ddmfile < 0)
	{
		empty_page(p);
		hprintf(p, "<H2>ERROR: Can't open file %s</H2>\n"
				"<P>%s", source, strerror(errno));
		end_page(p);
		close(fd);
		return 1;
	}

	read(ddmfile, &hdr, sizeof(struct I2O_MODULE_DESC_HDR));
	lseek(ddmfile, 0, SEEK_SET);

	hprintf(p, "<P><B>DDM information:</B>\n");
	hprintf(p, "<PRE>\n");
	hprintf(p, "%s", modinfo(&hdr));
	hprintf(p, "</PRE>\n");

	/* Initialize the sw_stat */ 
	if (stat(source, &sw_stat) < 0)
	{
		empty_page(p);
		hprintf(p, "<H2>ERROR: Can't stat file %s</H2>\n"
				"<P>%s", source, strerror(errno));
		end_page(p);
		close(fd);
		return 1;
	}
	swlen = sw_stat.st_size;

	buffer = mmap(0, (size_t)swlen, PROT_READ, MAP_SHARED, ddmfile, 0);
	if (buffer == MAP_FAILED)
	{
		empty_page(p);
		hprintf(p, "<H2>ERROR: Can't mmap file %s</H2>\n"
				"<P>%s", source, strerror(errno));
		end_page(p);
		close(fd);
		return 1;
	}

	maxfrag = (swlen/FRAGSIZE)+1;
	curfrag = 1;

	/* Initialize the structure i2o_sw_xfer */ 
	sw.iop      = iop;
	sw.flags    = 0;
	if (strcmp(loadType, "Install") == 0)
		sw.flags |= 0x01;
	if (strcmp(overrideMode, "Override") == 0)
		sw.flags |= 0x02;
	sw.sw_type  = modulevalue(moduleType);
	sw.sw_id    = hdr.orgId << 16 | hdr.moduleId; 
	sw.buf      = buffer;
	sw.swlen    = &swlen;
	sw.maxfrag  = &maxfrag;
	sw.curfrag  = &curfrag;

/*
	hprintf(p, "<PRE>\n");
	hprintf(p, "iop      = %d\n",sw.iop);
	hprintf(p, "flags    = 0x%02X\n",sw.flags);
	hprintf(p, "sw_type  = 0x%02X\n",sw.sw_type);
	hprintf(p, "sw_id    = 0x%08X\n",sw.sw_id);
	hprintf(p, "swlen    = %d\n", *(sw.swlen));
	hprintf(p, "buf      = %p\n", sw.buf);
	hprintf(p, "</PRE>\n");
*/

	/* download the file */
	hprintf(p, "<P>Downloading %d fragments: ", maxfrag);
	for (curfrag = 1; curfrag <= maxfrag; curfrag++) {
		if (ioctl(fd, I2OSWDL, &sw) == 0)
			hprintf(p, "%d .. ", curfrag);
		else
		{
			empty_page(p);
			hprintf(p, "<H2>ERROR: Can't download fragment %d/%d</H2>\n"
					"<P>%s", curfrag, maxfrag, strerror(errno));
			end_page(p);
			munmap(buffer, (size_t)swlen);
			close(fd);
			return 1;
		}
		sw.buf += FRAGSIZE;
	}
	
	hprintf(p, "complete\n");
	hprintf(p, "<P><A HREF=\"" CGIPATH "IOPDetails?IOP=%d\">Back to start</A>", iop);
	hprintf(p, "</BODY>\n");
	hprintf(p, "</HTML>");
	end_page(p);
	close(fd);
	
	return 0;
}

