/*
 *   I2O Configuration utility
 *
 *   IOP chooser
 *
 *   (C) Copyright 1999	University of Helsinki,
 *			Department of Computer Science
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation; either version
 *   2 of the License, or (at your option) any later version.
 *
 *   Author	Taneli Vhkangas <Taneli.Vahakangas@cs.Helsinki.FI>
 *
 */

#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>

#include <linux/i2o-dev.h>

#include "common.h"

#define MAXLEN 8192

int main(void)
{
	struct i2o_cmd_hrtlct cmd_lct;
	struct _i2o_lct *lct;
	unsigned int reslen;
	int fd;
	unsigned int iop_count = 0, i;
	unsigned char ioplist[MAX_I2O_CONTROLLERS];
	struct page *p;
	char *hostname = getenv("SERVER_NAME");

	p=(struct page *)malloc(sizeof(struct page));
	if (!p) 
	{
		printf("<H2>ERROR: No memory</H2><P>%s\n", strerror(errno));
		perror("malloc");
		return 1;
	}

	fd = open("/dev/i2o/ctl", O_RDONLY);
	if (fd < 0)
	{
		start_page(p);
		hprintf(p, "<H2>ERROR: Could not open connection to I<SUB>2</SUB>O platform.</H2>\n"
			"<P>%s",
			strerror(errno));
		end_page(p);
		perror("open");
		return 1;
	}

	start_page(p);
	hprintf(p, "<HTML>\n");
	hprintf(p, "<HEAD>\n");
	hprintf(p, "<TITLE>I2O Configuration on %s v." VERSION "</TITLE>\n",
			hostname ? hostname : "localhost");
	if (ioctl(fd, I2OGETIOPS, &ioplist) != 0)
	{
		empty_page(p);
		hprintf(p, "<H2>ERROR: Could not read the number of IOPs</H2>\n"
			"<P>%s", strerror(errno));
		end_page(p);
		perror("open");
		return 1;
	}

	for (i = 0; i<MAX_I2O_CONTROLLERS; i++)
		if (ioplist[i]) iop_count++;
 	
	if (iop_count == 1)
	{
		/* There is only one IOP in the system. Scotty beams us right there. */
		hprintf(p, "<META http-equiv=\"refresh\" content="
					"\"0; URL=http://%s" CGIPATH "IOPDetails?IOP=0\">\n",
					hostname ? hostname : "localhost");
		hprintf(p, "</HEAD>\n");

		hprintf(p, "<BODY>\n");
		hprintf(p, "<CENTER>\n");
		hprintf(p, "<FORM ACTION=\"" CGIPATH "IOPDetails\" METHOD=\"POST\">\n");
		hprintf(p, "<INPUT TYPE=\"HIDDEN\" NAME=\"IOP\" VALUE=\"0\">\n");
		hprintf(p, "<INPUT TYPE=\"SUBMIT\" VALUE=\"IOP0 Details\">\n");
		hprintf(p, "</FORM>\n");
		hprintf(p, "</CENTER>\n");
		hprintf(p, "</HTML>\n");
		end_page(p);
		
		return 0;
	} else {
		hprintf(p, "</HEAD>\n");
	}

	/* Ok, there are more than one IOP. Let user choose which she wants configure. */
	hprintf(p, "<BODY>\n");
	hprintf(p, "<CENTER>\n");

	hprintf(p, "<FONT SIZE=-1>There are %d IOPs in the system</FONT>\n",
			iop_count);
	hprintf(p, "<P>");

	lct = (void *)malloc(reslen);
	if (!lct)
	{
		empty_page(p);
		hprintf(p, "<H2>ERROR: Out of memory.\n"
			"<P>%s",
			strerror(errno));
		end_page(p);
		perror("open");
		return 1;
	}

	cmd_lct.resbuf = (void *)lct;

	hprintf(p, "<P>Please select IOP to configure:\n");
	hprintf(p, "<FORM ACTION=\"" CGIPATH "IOPDetails\" METHOD=\"POST\">\n");
	hprintf(p, "<SELECT NAME=\"IOP\">\n");
	for (i=0; i<MAX_I2O_CONTROLLERS; i++)
	{
		if (!ioplist[i])
			continue;

		reslen = MAXLEN;
		cmd_lct.reslen = &reslen;
		cmd_lct.iop = i;
		hprintf(p, "<OPTION VALUE=\"%d\">IOP %d ", i, i);
		if (ioctl(fd, I2OLCTGET, &cmd_lct) == 0)
		{
			if (lct->iop_flags & 1)
				hprintf(p, "(requesting configuration)");
		}
		else
			hprintf(p, "ERROR: Can't query LCT. %s",
					strerror(errno));
		hprintf(p, "\n");
	}
	hprintf(p, "</SELECT>\n");
	hprintf(p, "<INPUT TYPE=\"SUBMIT\" VALUE=\"Configure\">\n");
	hprintf(p, "</FORM>\n");
	hprintf(p, "</CENTER>\n");

	hprintf(p, "</BODY>\n");
	hprintf(p, "</HTML>\n");
	end_page(p);

	free(lct);
	close(fd);
	return 0;
}
