/*
 *   I2O Configuration utility
 *
 *   IOP main screen
 *
 *   (C) Copyright 1999	University of Helsinki,
 *			Department of Computer Science
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation; either version
 *   2 of the License, or (at your option) any later version.
 *
 *   Author	Taneli Vhkangas <Taneli.Vahakangas@cs.Helsinki.FI>
 *
 */ 

#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>

#include <linux/i2o-dev.h>

#include "common.h"

#define MAXLEN 16384

int main(void)
{
	struct i2o_driver *drivers;
	struct i2o_cmd_hrtlct cmd_lct;
	struct _i2o_lct *lct;
	unsigned int reslen = MAXLEN;
	int fd, iop, i;
	char *hostname, *query, *IOP;
	struct page *p;

	p = (struct page *)malloc(sizeof(struct page));
	if (!p) {
		printf("<H2>ERROR: No memory</H2><P>%s\n", strerror(errno));
		perror("malloc");
		return 1;
	}
	
	fd = open("/dev/i2o/ctl", O_RDONLY);
	if (fd < 0)
	{
		start_page(p);
		hprintf(p, "<H2>ERROR: Could not open connection to I<SUB>2</SUB>O platform.</H2>\n"
			"<P>%s",
			strerror(errno));
		end_page(p);
		return 1;
	}

	hostname = getenv("SERVER_NAME");
	query = getquery();
	IOP = decode(extract(query, "IOP"));
	if (!IOP)
	{
		start_page(p);
		hprintf(p, "<H2>ERROR: \"%.20s\" is not a valid IOP.</H2>\n",
			IOP ? IOP : "");
		end_page(p);
		return 1;
	}
	iop = atoi(IOP);

	start_page(p);
	hprintf(p, "<HTML>\n");
        hprintf(p, "<HEAD><TITLE>I2O Configuration for IOP%d on %s v." VERSION "</TITLE></HEAD>\n",
				iop, hostname ? hostname : "localhost");
	hprintf(p, "<BODY>\n");

/* Button to hardware configuration */
	hprintf(p, "<CENTER>\n");
	hprintf(p, "<H1>I<SUB>2</SUB>O Configuration for IOP%d on %s</H1>\n",
				iop, hostname ? hostname : "localhost");
	hprintf(p, "<FORM ACTION=\"" CGIPATH "Configure/%d/0/0\" METHOD=\"POST\">\n", iop);
	hprintf(p, "<TABLE BORDER=1>\n");
	hprintf(p, "<TR><TD COLSPAN=2 ALIGN=CENTER>");
	hprintf(p, "<INPUT TYPE=\"SUBMIT\" VALUE=\"Configure hardware\">");
	hprintf(p, "</TD></TR>\n");
	hprintf(p, "</TABLE>\n");
	hprintf(p, "</FORM>\n");

	/* Look at LCT, tell user if configuration requested. */
	lct = (void *)malloc(reslen);
	if (!lct)
	{
		empty_page(p);
		hprintf(p, "<H2>ERROR: Out of memory.\n"
			"<P>%s",
			strerror(errno));
		end_page(p);
		perror("malloc");
		return 1;
	}

	cmd_lct.resbuf = (void *)lct;
	cmd_lct.reslen = &reslen;
	cmd_lct.iop = iop;
	if (ioctl(fd, I2OLCTGET, &cmd_lct) == 0)
	{
		if (lct->iop_flags & 1) {
			hprintf(p, "<CENTER>\n"
					"<P>The TIDs: ");
			for (i=0; i<lct->table_size>>2; i++)
				if (lct->lct_entry[i].device_flags & 0x01)
					hprintf(p, "<B>[0x%04x]</B> ", i);
				else hprintf(p, "[0x%04x] ", i);
			hprintf(p, "<P>Entries marked with <B>emphasis</B>"
					" are requesting configuration.\n");
			hprintf(p, "<P>Please press \"Configure hardware\""
					" to see what is going on.\n");
			hprintf(p, "<P>Or, if you have recently downloaded DDMs"
					", please press Validate instead.\n");
			hprintf(p, "</CENTER>\n");
		}
	}
	else
	{
		hprintf(p, "ERROR: Can't query LCT. %s\n", strerror(errno));
		/* Incidentally, we know that the user most probably gave us a bogus IOP. */
		end_page(p);
		return 1;
	}

/* Configuration validation */
	hprintf(p, "<FORM ACTION=\"" CGIPATH "Validate\" METHOD=\"POST\">\n");
	hprintf(p, "<INPUT TYPE=\"HIDDEN\" NAME=\"iopdestvalidate\" VALUE=\"%d\">\n", iop);
	hprintf(p, "<TABLE BORDER=1>\n");
	hprintf(p, "<TR><TD COLSPAN=2>");
	hprintf(p, "    <INPUT TYPE=\"SUBMIT\" VALUE=\"Validate all suspect DDMs\">");
	hprintf(p, "</TD></TR>\n");
	hprintf(p, "</TABLE>\n");
	hprintf(p, "</FORM>\n");
	

/* Module download */
	hprintf(p, "<FORM ACTION=\"" CGIPATH "Download\" METHOD=\"POST\">\n");
	hprintf(p, "<INPUT TYPE=\"HIDDEN\" NAME=\"TargetIOP\" VALUE=\"%d\">\n", iop);
	hprintf(p, "<TABLE BORDER=1>\n");
	hprintf(p, "<TR><TD COLSPAN=2 ALIGN=CENTER><H2>Module download</H2></TD></TR>\n");
	hprintf(p, "<TR><TD WIDTH=\"20%%\"><B>Download method</B></TD>\n");
	hprintf(p, "    <TD><INPUT TYPE=\"RADIO\" NAME=\"LoadType\" VALUE=\"Install\" CHECKED>"
				"        Install into permanent store\n");
	hprintf(p, "        <BR><INPUT TYPE=\"RADIO\" NAME=\"LoadType\" VALUE=\"Load\">"
				"        Load into memory\n");
	hprintf(p, "</TD></TR>\n");
	hprintf(p, "<TR><TD WIDTH=\"20%%\"><B>Override mode</B></TD>\n");
	hprintf(p, "    <TD><INPUT TYPE=\"RADIO\" NAME=\"OverrideMode\" VALUE=\"Experimental\" CHECKED>"
				"        Tag as experimental, preserve old version\n");
	hprintf(p, "        <BR><INPUT TYPE=\"RADIO\" NAME=\"OverrideMode\" VALUE=\"Override\">"
				"        Tag as validated, delete old version\n");
	hprintf(p, "</TD></TR>\n");
	hprintf(p, "<TR><TD WIDTH=\"20%%\"><B>Module type</B></TD>\n");
	hprintf(p, "    <TD><SELECT NAME=\"ModuleType\">\n");
	for (i=0; i<NUM_MODULE_TYPES; i++)
	{
		hprintf(p, "        <OPTION%sVALUE=\"%s\">%s\n",
			i?" ":" SELECTED ", moduleinfo[i].moduletype, moduleinfo[i].moduledesc);
	}
	hprintf(p, "        </SELECT></TD></TR>\n");
	hprintf(p, "<TR><TD WIDTH=\"20%%\"><B>Source file</B></TD>\n");
	hprintf(p, "    <TD><INPUT TYPE=\"FILE\" VALUE=\"text/ddm\" NAME=\"Source\"></TD></TR>\n");
	hprintf(p, "<TR><TD COLSPAN=2 ALIGN=CENTER>");
	hprintf(p, "<INPUT TYPE=\"SUBMIT\" VALUE=\"Download\">(This is slow)");
	hprintf(p, "</TD></TR>\n");
	hprintf(p, "</TABLE>\n");
	hprintf(p, "</FORM>\n");

/* Module upload */
	hprintf(p, "<FORM ACTION=\"" CGIPATH "Upload\" METHOD=\"POST\">\n");
	hprintf(p, "<INPUT TYPE=\"HIDDEN\" NAME=\"IOP\" VALUE=\"%d\">\n", iop);
	hprintf(p, "<TABLE BORDER=1>\n");
	hprintf(p, "<TR><TD COLSPAN=2 ALIGN=CENTER><H2>Module upload</H2></TD></TR>\n");

	hprintf(p, "<TR><TD WIDTH=\"20%%\"><B>Module type</B></TD>\n");
	hprintf(p, "    <TD><SELECT NAME=\"ModuleType\">\n");
	for (i=0; i<NUM_MODULE_TYPES; i++)
	{
		hprintf(p, "        <OPTION%sVALUE=\"%s\">%s\n",
				i?" ":" SELECTED ", moduleinfo[i].moduletype, moduleinfo[i].moduledesc);
	}
	hprintf(p, "        </SELECT></TD></TR>\n");

	hprintf(p, "<TR><TD WIDTH=\"20%%\"><B>Software ID</B></TD>\n");
	hprintf(p, "    <TD>Only for DDMs, DDM parameter blocks and configuration dialogs\n"
				"        <P><SELECT NAME=\"SWID\">\n");
	drivers = getdst(fd, iop);
	if (drivers)
	{
		for (i=0; i<MAX_I2O_MODULES && i<drivers->row_count; i++)
		{
			hprintf(p, "        <OPTION VALUE=\"%d\">#%d 0x%04x 0x%04x %.28s\n",
					drivers->dst[i].stored_ddm_index,
					drivers->dst[i].stored_ddm_index,
					drivers->dst[i].i2o_vendor_id,
					drivers->dst[i].module_id,
					drivers->dst[i].module_name_version);
		}
		if (i==0) hprintf(p, "        <OPTION VALUE=\"ERROR\">No drivers stored\n");
	}
	else
	{
		hprintf(p, "        <OPTION VALUE=\"ERROR\">"
					"ERROR: Can't query driver store table\n");
	}
	hprintf(p, "        </SELECT></TD></TR>\n");
	/* I2O v2.0 addition */
	hprintf(p, "<TR><TD WIDTH=\"20%%\"><B>Version</B></TD>\n");
	hprintf(p, "    <TD><INPUT TYPE=\"RADIO\" NAME=\"SWVersion\" VALUE=\"Active\" CHECKED>Active\n");
	hprintf(p, "        <INPUT TYPE=\"RADIO\" NAME=\"SWVersion\" VALUE=\"Valid\">Valid\n");
	hprintf(p, "        <INPUT TYPE=\"RADIO\" NAME=\"SWVersion\" VALUE=\"Obsolete\">Obsolete\n");
	hprintf(p, "        <INPUT TYPE=\"RADIO\" NAME=\"SWVersion\" VALUE=\"Suspect\">Suspect\n");
	hprintf(p, "</TD></TR>\n");
	hprintf(p, "<TR><TD COLSPAN=2 ALIGN=CENTER>");
	hprintf(p, "<INPUT TYPE=\"SUBMIT\" VALUE=\"Upload\">");
	hprintf(p, "</TD></TR>\n");
	hprintf(p, "</TABLE>\n");
	hprintf(p, "</FORM>\n");

/* Module removal */
	hprintf(p, "<FORM ACTION=\"" CGIPATH "Remove\" METHOD=\"POST\">\n");
	hprintf(p, "<INPUT TYPE=\"HIDDEN\" NAME=\"IOP\" VALUE=\"%d\">\n", iop);
	hprintf(p, "<TABLE BORDER=1>\n");
	hprintf(p, "<TR><TD COLSPAN=2 ALIGN=CENTER><H2>Module removal</H2></TD></TR>\n");

	hprintf(p, "<TR><TD WIDTH=\"20%%\"><B>Module type</B></TD>\n");
	hprintf(p, "    <TD><SELECT NAME=\"ModuleType\">\n");
	for (i=0; i<NUM_MODULE_TYPES; i++)
	{
		if (moduleinfo[i].value == 0x11) continue; /* There's no point in removing IRTOS. */
		hprintf(p, "        <OPTION%sVALUE=\"%s\">%s\n",
				i?" ":" SELECTED ", moduleinfo[i].moduletype, moduleinfo[i].moduledesc);
	}
	hprintf(p, "        </SELECT></TD></TR>\n");

	hprintf(p, "<TR><TD WIDTH=\"20%%\"><B>Software ID</B></TD>\n");
	hprintf(p, "    <TD>Only for DDMs, DDM parameter blocks and configuration dialogs\n"
				"        <P><SELECT NAME=\"SWID\">\n");
	drivers = getdst(fd, iop);
	if (drivers)
	{
		for (i=0; i<MAX_I2O_MODULES && i<drivers->row_count; i++)
		{
			hprintf(p, "        <OPTION VALUE=\"%d\">#%d 0x%04x 0x%04x %.28s\n",
					drivers->dst[i].stored_ddm_index,
					drivers->dst[i].stored_ddm_index,
					drivers->dst[i].i2o_vendor_id,
					drivers->dst[i].module_id,
					drivers->dst[i].module_name_version);
		}
		if (i==0) hprintf(p, "        <OPTION VALUE=\"ERROR\">No drivers stored\n");
	}
	else
	{
		hprintf(p, "        <OPTION VALUE=\"ERROR\">"
					"ERROR: Can't query driver store table\n");
	}
	hprintf(p, "        </SELECT></TD></TR>\n");
	/* I2O v2.0 addition */
	hprintf(p, "<TR><TD WIDTH=\"20%%\"><B>Version</B></TD>\n");
	hprintf(p, "    <TD><INPUT TYPE=\"RADIO\" NAME=\"SWVersion\" VALUE=\"Active\" CHECKED>Active\n");
	hprintf(p, "        <INPUT TYPE=\"RADIO\" NAME=\"SWVersion\" VALUE=\"Valid\">Valid\n");
	hprintf(p, "        <INPUT TYPE=\"RADIO\" NAME=\"SWVersion\" VALUE=\"Obsolete\">Obsolete\n");
	hprintf(p, "        <INPUT TYPE=\"RADIO\" NAME=\"SWVersion\" VALUE=\"Suspect\">Suspect\n");
	hprintf(p, "</TD></TR>\n");
	hprintf(p, "<TR><TD COLSPAN=2 ALIGN=CENTER>");
	hprintf(p, "<INPUT TYPE=\"SUBMIT\" VALUE=\"Remove\">");
	hprintf(p, "</TD></TR>\n");
	hprintf(p, "</TABLE>\n");
	hprintf(p, "</FORM>\n");

	hprintf(p, "</CENTER>\n");
	hprintf(p, "</BODY>\n");
	hprintf(p, "</HTML>\n");

	end_page(p);

	return 0;
}
