/*
 *   I2O Configuration utility
 *
 *   Software remove
 *
 *   (C) Copyright 1999	University of Helsinki,
 *			Department of Computer Science
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation; either version
 *   2 of the License, or (at your option) any later version.
 *
 *   Author	Taneli Vhkangas <Taneli.Vahakangas@cs.Helsinki.FI>
 *
 */

#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>

#include <linux/i2o-dev.h>

#include "common.h"

int main(void)
{
	struct i2o_sw_xfer sw;
	struct i2o_driver *drivers;
	size_t swlen;
	int i, fd, iop, driver_id;
	char *query, *IOP, *Module, *SWID, *SWVersion;
	struct page *p;

	p = (struct page *)malloc(sizeof(struct page));
	if (!p) {
		error_page("<H2>ERROR: Out of memory</H2><P>%s.\n", strerror(errno));
		return 1;
	}

	fd = open("/dev/i2o/ctl", O_RDONLY);
	if (fd < 0)
	{
		start_page(p);
		hprintf(p, "<H2>ERROR: Could not open connection to I<SUB>2</SUB>O platform.</H2>\n"
			"<P>%s", strerror(errno));
		end_page(p);
		perror("open");
		return 1;
	}

	query = getquery();
	Module    = decode(extract(query, "ModuleType"));
	IOP       = decode(extract(query, "IOP"));
	SWID      = decode(extract(query, "SWID"));
	SWVersion = decode(extract(query, "SWVersion"));
	iop       = atoi(IOP);

	sw.sw_type = modulevalue(Module);

	if (sw.sw_type == 0x00 || sw.sw_type == 0x01 || sw.sw_type == 0x02)
	{
		driver_id = atoi(SWID);
		drivers = getdst(fd, iop);
		if (drivers)
		{
			for (i=0; i<MAX_I2O_CONTROLLERS; i++)
				if (drivers->dst[i].stored_ddm_index == driver_id)
					break;
		}

		if (drivers->dst[i].stored_ddm_index != driver_id)
		{
			start_page(p);
			hprintf(p, "<H2>ERROR: Can't find specified software module.\n");
			end_page(p);
			return 1;
		}

		swlen = drivers->dst[i].module_size;
		sw.sw_id = drivers->dst[i].i2o_vendor_id << 16 | drivers->dst[i].module_id;
	}
	else
	{
		swlen = 0;
	}

	sw.iop    = iop;
	if (!strcmp(SWVersion, "Active"))   sw.flags = 0x00;
	if (!strcmp(SWVersion, "Valid"))    sw.flags = 0x02;
	if (!strcmp(SWVersion, "Obsolete")) sw.flags = 0x04;
	if (!strcmp(SWVersion, "Suspect"))  sw.flags = 0x08;
	sw.flags  = 0x00; /* It's actually specified only in spec v2.0 */
	sw.buf    = NULL;
	sw.swlen  = &swlen;

	if (ioctl(fd, I2OSWDEL, &sw) != 0)
	{
		start_page(p);
		hprintf(p, "<H2>ERROR: Can't remove software. %s.\n"
				"<P>sw_id=%08x, type=%08x"
				", flags=%08x, swlen=%d\n",
				strerror(errno), sw.sw_id, sw.sw_type,
				sw.flags, *(sw.swlen));
		end_page(p);
		return 1;
	}

	start_page(p);
	hprintf(p, "<H2>Software has been removed from the IOP.</H2>\n");
	end_page(p);
	return 0;
}
