/*
 *   I2O Configuration utility
 *
 *   Software upload
 *
 *   (C) Copyright 1999	University of Helsinki,
 *                      Department of Computer Science
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation; either version
 *   2 of the License, or (at your option) any later version.
 *
 *   Author    Taneli Vhkangas <Taneli.Vahakangas@cs.Helsinki.FI>
 *
 */

#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>

#include <linux/i2o-dev.h>

#include "common.h"

#define MAXLEN 8192
#define FRAGSIZE 8192

extern struct i2o_driver *getdst(int, int);

int main(void)
{
	struct i2o_sw_xfer sw;
	struct i2o_driver *drivers;
	size_t swlen;
	int i, fd, iop, driver_id;
	unsigned int curfrag = 0, maxfrag = 0;
	char *query, *IOP, *Module, *SWID, *SWVersion;
	struct page *p;
	char *software;

	p = (struct page *)malloc(sizeof(struct page));
	if (!p) {
		error_page("<H2>ERROR: Out of memory</H2><P>%s.", strerror(errno));
		return 1;
	}

	fd = open("/dev/i2o/ctl", O_RDONLY);
	if (fd < 0)
	{
		start_page(p);
		hprintf(p, "<H2>ERROR: Could not open connection to I<SUB>2</SUB>O platform.</H2>\n"
			"<P>%s", strerror(errno));
		end_page(p);
		perror("open");
		return 1;
	}

	query = getquery();
	Module    = decode(extract(query, "ModuleType"));
	IOP       = decode(extract(query, "IOP"));
	SWID      = decode(extract(query, "SWID"));
	SWVersion = decode(extract(query, "SWVersion"));
	iop       = atoi(IOP);

	sw.sw_type = modulevalue(Module);

	if (sw.sw_type == 0x00 || sw.sw_type == 0x01 || sw.sw_type == 0x02) /* Its a DDM or DDM_MPB */
	{
		driver_id = atoi(SWID);
		drivers = getdst(fd, iop);
		if (drivers)
		{
			for (i=0; i<MAX_I2O_CONTROLLERS; i++)
				if (drivers->dst[i].stored_ddm_index == driver_id)
					break;
		}

		if (drivers->dst[i].stored_ddm_index != driver_id)
		{
			start_page(p);
			hprintf(p, "<H2>ERROR: Can't find specified software module.\n");
			end_page(p);
			return -1;
		}

		swlen   = drivers->dst[i].module_size;
		maxfrag = (swlen >> 13) + 1;
		sw.sw_id   = drivers->dst[i].i2o_vendor_id << 16 | drivers->dst[i].module_id;
	}
	else
	{
		swlen = 0;
		maxfrag = -1;
	}

	software = (char *)malloc(swlen);
	if (!software)
	{
		start_page(p);
		hprintf(p, "<H2>ERROR: Out of memory</H2>\n");
		end_page(p);
		return 1;
	}

	sw.iop     = iop;
	if (!strcmp(SWVersion, "Active"))   sw.flags = 0x00;
	if (!strcmp(SWVersion, "Valid"))    sw.flags = 0x02;
	if (!strcmp(SWVersion, "Obsolete")) sw.flags = 0x04;
	if (!strcmp(SWVersion, "Suspect"))  sw.flags = 0x08;
	sw.flags   = 0x00;  /* DownloadFlags is only specified in I2O v2.0 */
	sw.buf     = software;
	sw.swlen   = &swlen;
	sw.maxfrag = &maxfrag;
	sw.curfrag = &curfrag;

	if (maxfrag == -1) /* We don't know the software size. */
	{
		start_page(p);
		hprintf(p, "<H2>TODO: Only DDM uploading implemented.\n");
		end_page(p);
		return 1;
	}
	for (curfrag = 1; curfrag <= maxfrag; curfrag++)
	{
		if (ioctl(fd, I2OSWUL, &sw) != 0)
		{
			start_page(p);
			hprintf(p, "<H2>ERROR: Can't upload fragment %d/%d. %s</H2>\n"
				"<P>swlen = %d\n", curfrag, maxfrag, strerror(errno), swlen);
			end_page(p);
			return 1;
		}
		sw.buf += FRAGSIZE;
	}
	start_page(p);
	change_type(p, "application/octet-stream");
	write_page(p, software, swlen);
	end_page(p);
	return 0;
}
