/*
 *   I2O Configuration utility
 *
 *   Common parts
 *
 *   (C) Copyright 1999 University of Helsinki,
 *                      Department of Computer Science
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation; either version
 *   2 of the License, or (at your option) any later version.
 *
 *   Author     Taneli Vhkangas <Taneli.Vahakangas@cs.Helsinki.FI>
 *
 */
 
#ifndef __COMMON_H_
#define __COMMON_H_

/* #include <stdio.h> */

/*
 * CGI handling part
 */

extern void html_error(const char *);
extern char *extract(const char *, const char *);
extern char *encode(const char *);
extern char *decode(const char *);
extern char *getquery(void);

extern int querylen;





/*
 * Parts pertaining to HTML handling
 */

char *prologue;
char *epilogue;

extern FILE *output;
extern int outputfd;

struct page {
	FILE *output;
	char *str;
	char type[40];
	int binary;
	int offset;
	int outputfd;
};

extern void start_page(struct page *);
extern void error_page(const char *format, ...);
extern void change_type(struct page *, char *);
extern int hprintf(struct page *, const char *format, ...);
extern void write_page(struct page *, char *, unsigned int);
extern void empty_page(struct page *);
extern void end_page(struct page *);





/*
 * I2O specific parts
 */

/* Entries in moduleinfo */
#define NUM_MODULE_TYPES 9

/* Max number of drivers */
#define MAX_I2O_MODULES 64

struct I2O_MODULE_DESC_HDR
{
	unsigned int         headerSize;         /* size of this header and tables */
	unsigned short       orgId;              /* I2O organization ID */
	unsigned short       moduleId;           /* assigned to vendor of module */
	unsigned short       day;                /* ascii 4 digit day DDM produced */
	unsigned short       month;              /* ascii 4 digit month DDM produced */
	unsigned int         year;               /* ascii 4 digit year DDM produced */
	unsigned char        i2oVersion;         /* I2O version info */
	unsigned char        majorCapabilities;  /* capbilities bits */
	unsigned short       reserved;           /* reserved */
	unsigned int         codeSize;           /* text/data/bss */
	unsigned int         tableOffset;        /* offset to numTables */
	unsigned int         memoryReq;          /* pre-attach memory requiremets */
	unsigned int         memoryPreferred;    /* additional desired */
	char                 moduleVersion[4];   /* 4 ascii characters */
	unsigned char        processorType;      /* IOP processor type */
	unsigned char        processVersion;     /* IOP processor type */
	unsigned char        objCodeFormat;      /* DDM object module format */
	unsigned char        reserved1;          /* reserved */
	unsigned int         numTables;          /* number of descriptor tables */
	char                 moduleInfo[24];     /* ascii string name */
};

struct mod_info {
	char *moduletype;
	unsigned char value;
	char *moduledesc;
};

extern struct mod_info moduleinfo[NUM_MODULE_TYPES];

typedef struct _i2o_driver_store {
	u16 stored_ddm_index;
	u8  module_type;
	u8  reserved;
	u16 i2o_vendor_id;
	u16 module_id;
	u8  module_name_version[28];
	u8  date[8];
	u32 module_size;
	u32 mpb_size;
	u32 module_flags;
} i2o_driver_store_table;

struct i2o_driver {
	u16 result_count;
	u16 pad;
	u16 block_size;
	u8  block_status;
	u8  error_info_size;
	u16 row_count;
	u16 more_flag;
	i2o_driver_store_table dst[MAX_I2O_MODULES];
};

extern unsigned char modulevalue(const char *);
extern struct i2o_driver *getdst(int, int);
extern char *modinfo(struct I2O_MODULE_DESC_HDR *);

#endif /* __COMMON_H */
